<?php

namespace Drupal\smart_content_block\Plugin\smart_content\VariationSetType;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\smart_content\Form\SmartVariationSetForm;
use Drupal\smart_content\VariationSetType\VariationSetTypeBase;

/**
 * @SmartVariationSetType(
 *   id = "block",
 *   label = @Translation("Block"),
 * )
 */
class Block extends VariationSetTypeBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $wrapper_id = Html::getUniqueId('variation-set-wrapper');
    $wrapper_items_id = Html::getUniqueId('variation-set-items-wrapper');

    $form['variations_config'] = [
      '#type' => 'container',
      '#title' => 'Variations',
      '#tree' => TRUE,
      '#prefix' => '<div id="' . $wrapper_id . '" class="variations-container">',
      '#suffix' => '</div>',
    ];
    $form['variations_config']['variation_items'] = [
      '#type' => 'table',
      '#header' => [t('Variations'), t('Weight'), t('')],
      '#prefix' => '<div id="' . $wrapper_items_id . '" class="variations-container-items">',
      '#suffix' => '</div>',
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => $wrapper_items_id . '-order-weight',
        ],
      ],
    ];

    $i = 0;
    foreach ($this->entity->getVariations() as $variation_id => $variation) {
      $i++;

      SmartVariationSetForm::pluginForm($variation, $form, $form_state, [
        'variations_config',
        'variation_items',
        $variation_id,
        'plugin_form',
      ]);

      $form['variations_config']['variation_items'][$variation_id]['plugin_form']['#type'] = 'fieldset';
      $form['variations_config']['variation_items'][$variation_id]['plugin_form']['#title'] = 'Variation ' . $i;
      $form['variations_config']['variation_items'][$variation_id]['plugin_form']['#attributes']['class'][] = 'variation-container';
      $form['variations_config']['variation_items'][$variation_id]['#attributes']['class'][] = 'draggable';
      $form['variations_config']['variation_items'][$variation_id]['#weight'] = $variation->getWeight();

      $form['variations_config']['variation_items'][$variation_id]['weight'] = [
        '#type' => 'weight',
        '#title' => 'Weight',
        '#title_display' => 'invisible',
        '#attributes' => ['class' => [$wrapper_items_id . '-order-weight']],
      ];

      $form['variations_config']['variation_items'][$variation_id]['remove_variation'] = [
        '#type' => 'submit',
        '#value' => t('Remove Variation'),
        '#name' => 'remove_variation__' . $variation_id,
        '#submit' => [[$this, 'removeElementVariation']],
        '#attributes' => [
          'class' => [
            'align-right',
            'remove-variation',
            'remove-button',
          ],
        ],
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => [$this, 'removeElementVariationAjax'],
          'wrapper' => $wrapper_id,
        ],
      ];
      $disabled = ($this->entity->getDefaultVariation() && $this->entity->getDefaultVariation() != $variation_id) ? 'disabled' : '';
      $form['variations_config']['variation_items'][$variation_id]['plugin_form']['additional_settings'] = [
        '#type' => 'container',
        '#weight' => 10,
        '#attributes' => [
          'class' => ['variation-additional-settings-container'],
          'disabled' => [$disabled],
        ],
      ];
      $form['variations_config']['variation_items'][$variation_id]['plugin_form']['additional_settings']['default_variation'] = [
        '#type' => 'checkbox',
        '#attributes' => [
          'class' => ['smart-variations-default-' . $variation_id],
          'disabled' => [$disabled],
        ],
        '#title' => 'Set as default variation',
        '#default_value' => $this->entity->getDefaultVariation() == $variation_id,
      ];
    }

    $form['variations_config']['add_variation'] = [
      '#type' => 'submit',
      '#value' => t('Add Variation'),
      '#submit' => [[$this, 'addElementVariation']],
      '#limit_validation_errors' => [],
      '#ajax' => [
        'callback' => [$this, 'addElementVariationAjax'],
        'wrapper' => $wrapper_id,
      ],
    ];


    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    foreach ($this->entity->getVariations() as $variation_id => $variation) {
      SmartVariationSetForm::pluginFormValidate($variation, $form, $form_state, [
        'variations_config',
        'variation_items',
        $variation_id,
        'plugin_form',
      ]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->attachTableVariationWeight($form_state->getValues()['variations_config']['variation_items']);
    foreach ($this->entity->getVariations() as $variation_id => $variation) {
      SmartVariationSetForm::pluginFormSubmit($variation, $form, $form_state, [
        'variations_config',
        'variation_items',
        $variation_id,
        'plugin_form',
      ]);
    }
    $this->attachFormValues($form_state->getValues()['variations_config']['variation_items']);
  }


  public function attachFormValues($values) {
    foreach ($this->entity->getVariations() as $variation) {
      // Attaching default_variation value to variation config.
      if (isset($values[$variation->id()]['plugin_form']['additional_settings']['default_variation'])) {
        if ($values[$variation->id()]['plugin_form']['additional_settings']['default_variation']) {
          $this->entity->setDefaultVariation($variation->id());
        }
        else {
          if ($variation->id() == $this->entity->getDefaultVariation()) {
            $this->entity->setDefaultVariation('');
          }
        }
      }
    }
  }

  public function addElementVariation(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $items = array_slice($button['#parents'], 0, -1);
    $items[] = 'variation_items';
    $values = NestedArray::getValue($form_state->getUserInput(), $items);
    $this->entity->addVariation(\Drupal::service('plugin.manager.smart_content.variation')
      ->createInstance('variation_block', [], $this->entity));
    $form_state->setRebuild();
  }

  public function addElementVariationAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    // Go one level up in the form, to the widgets container.
    return NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -1));
  }

  public function removeElementVariation(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $items = array_slice($button['#parents'], 0, -3);
    $items[] = 'variation_items';
    $values = NestedArray::getValue($form_state->getUserInput(), $items);
    $this->attachTableVariationWeight($values);
    list($action, $name) = explode('__', $form_state->getTriggeringElement()['#name']);
    $this->entity->removeVariation($name);
    $form_state->setRebuild();
  }

  public function removeElementVariationAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    // Go one level up in the form, to the widgets container.
    return NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -3));
  }
}
