<?php

namespace Drupal\smart_content\Variation;

use Drupal\Component\Plugin\ConfigurablePluginInterface;
use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\smart_content\Condition\ConditionInterface;
use Drupal\smart_content\Condition\ConditionManager;
use Drupal\smart_content\Entity\SmartVariationSet;
use Drupal\smart_content\Reaction\ReactionInterface;
use Drupal\smart_content\Reaction\ReactionManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;


/**
 * Base class for Smart variation plugins.
 */
abstract class VariationBase extends PluginBase implements VariationInterface, ConfigurablePluginInterface, PluginFormInterface, ContainerFactoryPluginInterface {

  use DependencySerializationTrait;

  protected $weight;

  protected $conditionManager;

  protected $reactionManager;

  protected $conditions;

  protected $reactions;


  /**
   * Creates an instance of the plugin.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.smart_content.condition'),
      $container->get('plugin.manager.smart_content.reaction')
    );
  }

  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConditionManager $condition_manager, ReactionManager $reaction_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->conditionManager = $condition_manager;
    $this->reactionManager = $reaction_manager;
  }

  public function id() {
    return isset($this->configuration['id']) ? $this->configuration['id'] : NULL;
  }

  public function setId($id) {
    $configuration = $this->getConfiguration();
    $configuration['id'] = $id;
    $this->setConfiguration($configuration);
  }

  /**
   * @inheritdoc
   */
  public function defaultConfiguration() {
    return [
      'id' => $this->id(),
      'plugin_id' => $this->getPluginId(),
      'weight' => $this->getWeight(),
    ];
  }

  /**
   * @inheritdoc
   */
  public function calculateDependencies() {
    // TODO: Implement calculateDependencies() method.
  }


  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = $configuration + $this->defaultConfiguration();
  }

  public function setWeight($weight) {
    $configuration = $this->getConfiguration();
    $configuration['weight'] = $weight;
    $this->setConfiguration($configuration);
  }

  public function getWeight() {
    return isset($this->configuration['weight']) ? $this->configuration['weight'] : 0;
  }

  /**
   * @param \Drupal\smart_content\Condition\ConditionInterface $condition
   */
  public function addCondition(ConditionInterface $condition) {
    if ($condition->id() === NULL) {
      $condition->setId(SmartVariationSet::generateUniquePluginId($condition, array_keys($this->getConditions())));
    }
    $this->conditions[$condition->id()] = $condition;

  }

  /**
   * @return array
   */
  public function getConditions() {
    if (!isset($this->conditions)) {
      $this->conditions = [];
      foreach ($this->getConditionsFromSettings() as $plugin) {
        $this->addCondition($plugin);
      }
    }
    return $this->conditions;
  }

  /**
   * @param $id
   *
   * @return mixed
   */
  public function getCondition($id) {
    foreach ($this->getConditions() as $condition) {
      if ($condition->id() == $id) {
        return $condition;
      }
    }
    return NULL;
  }

  /**
   * @param $id
   */
  public function removeCondition($id) {
    unset($this->conditions[$id]);
  }


  protected function getConditionsFromSettings() {
    $plugins = [];
    if (!empty($this->getConfiguration()['conditions_settings'])) {
      foreach ($this->getConfiguration()['conditions_settings'] as $id => $value) {
        $plugins[] = $this->conditionManager->createInstance($value['plugin_id'], $value);
      }
    }
    return $plugins;
  }


  public function sortConditions() {
    if ($this->getConditions()) {
      uasort($this->conditions, function ($first, $second) {
        return $first->getWeight() > $second->getWeight();
      });
    }
  }

  public function addReaction(ReactionInterface $reaction) {
    if ($reaction->id() === NULL) {
      $reaction->setId(SmartVariationSet::generateUniquePluginId($reaction, array_keys($this->getReactions())));
    }
    //@todo: find better way to do this.
    $this->reactions[$reaction->id()] = $reaction;

  }

  /**
   * @return array
   */
  public function getReactions() {
    if (!isset($this->reactions)) {
      $this->reactions = [];
      foreach ($this->getReactionsFromSettings() as $plugin) {
        $this->addReaction($plugin);
      }
    }
    return $this->reactions;
  }

  /**
   * @param $id
   *
   * @return mixed
   */
  public function getReaction($id) {
    foreach ($this->getReactions() as $reaction) {
      if ($reaction->id() == $id) {
        return $reaction;
      }
    }
    return NULL;
  }

  /**
   * @param $id
   */
  public function removeReaction($id) {
    unset($this->reactions[$id]);
  }


  protected function getReactionsFromSettings() {
    $plugins = [];
    if (!empty($this->getConfiguration()['reactions_settings'])) {
      foreach ($this->getConfiguration()['reactions_settings'] as $id => $value) {
        $plugins[] = $this->reactionManager->createInstance($value['plugin_id'], $value, $this->entity);
      }
    }
    return $plugins;
  }


  public function sortReactions() {
    if ($this->getReactions()) {
      uasort($this->reactions, function ($first, $second) {
        return $first->getWeight() > $second->getWeight();
      });
    }
  }

  public function writeChangesToConfiguration() {
    $configuration = $this->getConfiguration();
    // Ensure conditions are loaded from config, then wipe settings.
    // @todo: Should this be replaced with a getter/setter?
    $this->getConditions();
    $configuration['conditions_settings'] = [];
    foreach ($this->getConditions() as $condition) {
      $condition->writeChangesToConfiguration();
      $configuration['conditions_settings'][] = $condition->getConfiguration();
    }
    // Ensure reactions are loaded from config, then wipe settings.
    // @todo: Should this be replaced with a getter/setter?
    $this->getReactions();
    $configuration['reactions_settings'] = [];
    foreach ($this->getReactions() as $reaction) {
      $reaction->writeChangesToConfiguration();
      $configuration['reactions_settings'][] = $reaction->getConfiguration();
    }
    $this->setConfiguration($configuration);
  }


  public function getLibraries() {
    $libraries = [];

    foreach ($this->getConditions() as $condition) {
      $libraries = array_unique(array_merge($libraries, $condition->getLibraries()));
    }

    return $libraries;
  }


  public function getAttachedSettings() {
    $condition_settings = [];
    foreach ($this->getConditions() as $condition) {
      $condition_settings[] = $condition->getAttachedSettings();
    }
    return [
      'id' => $this->id(),
      'conditions' => $condition_settings,
    ];
  }


  function getResponse($context = []) {
    $response = new AjaxResponse();
    $content = [];
    foreach ($this->getReactions() as $reaction) {
      $reaction->buildResponse($response);
      if ($reaction_content = $reaction->getResponseContent($context)) {
        $content[] = $reaction_content;
      }
    }
    if (!empty($content)) {
      $response->addCommand(new ReplaceCommand($this->entity->getDecisionAgent()
        ->getResponseTarget(), $content));
    }
    return $response;
  }



  public function attachTableConditionWeight($values) {
    foreach ($this->getConditions() as $condition) {
      if (isset($values[$condition->id()]['weight'])) {
        $condition->setWeight($values[$condition->id()]['weight']);
      }
    }
    $this->sortConditions();
  }



  public function attachTableReactionWeight($values) {
    foreach ($this->getReactions() as $reaction) {
      if (isset($values[$reaction->id()]['weight'])) {
        $reaction->setWeight($values[$reaction->id()]['weight']);
      }
    }
    $this->sortReactions();
  }
}

